<?php

declare(strict_types=1);

use App\Entity\DrinkType;
use App\Entity\PropertyChangeLog;
use App\Service\DrinkType\GetStockHistory;
use Doctrine\ORM\EntityManagerInterface;

test('it returns empty array for unsaved drink type', function (): void {
    $drinkType = new DrinkType();
    $drinkType->setName('Test Drink');

    $getStockHistory = $this->getContainer()->get(GetStockHistory::class);
    $result = $getStockHistory($drinkType);

    expect($result)->toBeArray();
    expect($result)->toBeEmpty();
});

test('it returns stock history for a drink type', function (): void {
    // Create a drink type
    $drinkType = new DrinkType();
    $drinkType->setName('Test Drink');
    $drinkType->setCurrentStock(10);

    $em = $this->getContainer()->get(EntityManagerInterface::class);
    $em->persist($drinkType);
    $em->flush();

    // Change the current stock to create logs
    $drinkType->setCurrentStock(15);
    $em->persist($drinkType);
    $em->flush();

    $drinkType->setCurrentStock(5);
    $em->persist($drinkType);
    $em->flush();

    // Get the stock history
    $getStockHistory = $this->getContainer()->get(GetStockHistory::class);
    $result = $getStockHistory($drinkType);

    // Verify the result
    expect($result)->toBeArray();
    expect($result)->toHaveCount(3);

    // Verify that all logs are for currentStock
    foreach ($result as $log) {
        expect($log)->toBeInstanceOf(PropertyChangeLog::class);
        expect($log->getPropertyName())->toBe('currentStock');
        expect($log->getEntityClass())->toBe(DrinkType::class);
        expect($log->getEntityId())->toBe($drinkType->getId());
    }

    // Verify the values in order (oldest first)
    expect($result[0]->getNewValue())->toBe('10');
    expect($result[1]->getNewValue())->toBe('15');
    expect($result[2]->getNewValue())->toBe('5');
});

test('it only returns logs for the specified drink type', function (): void {
    $em = $this->getContainer()->get(EntityManagerInterface::class);

    // Create first drink type
    $drinkType1 = new DrinkType();
    $drinkType1->setName('Drink 1');
    $drinkType1->setCurrentStock(10);
    $em->persist($drinkType1);
    $em->flush();

    // Create second drink type
    $drinkType2 = new DrinkType();
    $drinkType2->setName('Drink 2');
    $drinkType2->setCurrentStock(20);
    $em->persist($drinkType2);
    $em->flush();

    // Get history for first drink type
    $getStockHistory = $this->getContainer()->get(GetStockHistory::class);
    $result = $getStockHistory($drinkType1);

    // Verify we only get logs for the first drink type
    expect($result)->toHaveCount(1);
    expect($result[0]->getEntityId())->toBe($drinkType1->getId());
    expect($result[0]->getNewValue())->toBe('10');
});
